<?php
/**
 * @package		Joomla.Site
 * @subpackage	com_users
 * @copyright	Copyright (C) 2005 - 2013 Open Source Matters, Inc. All rights reserved.
 * @license		GNU General Public License version 2 or later; see LICENSE.txt
 */

defined('_JEXEC') or die;

jimport('joomla.application.component.view');
JHTML::_('behavior.modal'); 

/**
 * Profile view class for Users.
 *
 * @package		Joomla.Site
 * @subpackage	com_users
 * @since		1.6
 */
class ZWeatherViewLocation extends JViewLegacy
{
	protected $data;
	protected $form;
	protected $params;
	protected $state;

	/**
	 * Method to display the view.
	 *
	 * @param	string	$tpl	The template file to include
	 * @since	1.6
	 */
	public function display($tpl = null)
	{
		
		// Get the view data.
		$this->data		= $this->get('Data');
		$this->form		= $this->get('Form');
		$this->state	= $this->get('State');
		$this->params	= $this->state->get('params');

		// Check for errors.
		if (count($errors = $this->get('Errors'))) {
			JError::raiseError(500, implode('<br />', $errors));
			return false;
		}

		// Check for layout override
		$active = JFactory::getApplication()->getMenu()->getActive();
		if (isset($active->query['layout'])) {
			$this->setLayout($active->query['layout']);
		}

		//Escape strings for HTML output
		$this->pageclass_sfx = htmlspecialchars($this->params->get('pageclass_sfx'));

		$this->prepareDocument();
		JHtml::stylesheet("components/com_zweather/assets/css/zweather.css");
		JHtml::stylesheet("components/com_zweather/assets/css/custom.css");
		
		parent::display($tpl);
	}

	/**
	 * Prepares the document
	 *
	 * @since	1.6
	 */
	protected function prepareDocument()
	{
		$app		= JFactory::getApplication();
		$menus		= $app->getMenu();
		$user		= JFactory::getUser();
		$title 		= null;

		// Because the application sets a default page title,
		// we need to get it from the menu item itself
		$menu = $menus->getActive();
		if($menu) {
			$this->params->def('page_heading', $this->params->get('page_title', $user->name));
		} else {
			$this->params->def('page_heading', JText::_('COM_ZWEATHER_WEATHER'));
		}

		$title = $this->params->get('page_title', '');
		if (empty($title)) {
			$title = $app->getCfg('sitename');
		}
		$this->document->setTitle($title);

	}
	
	function displayForecast24($options){
		$periods = $options->get('num_periods24', 6);	
		$m = $this->getModel();
		$id = $options->get("id",JRequest::getVar("id"));
		$f = $m->getForecast($id);
		if(substr($f,0,6) != '<dwml '){
			echo $f;
			return;
		} 
		$xml = JFactory::getXML($f,false);
		$helper = new ZWeatherHelper();
		$days = $xml->data->{'time-layout'}->{'start-valid-time'};
		$icons = $xml->data->parameters->{'conditions-icon'}->{'icon-link'};
		$conditions = $xml->data->parameters->weather->{'weather-conditions'};
		$temps = $xml->data->parameters->temperature;
		$hitemps = $temps[0]->value;
		$lotemps = $temps[1]->value;
		echo "<div class='forecast f24'>\n";
		echo "<ul>\n";
		for($i = 0; $i < $periods; $i++){
			echo "<li class='p{$periods}'>\n";
			$day = $helper->convertDate($days[$i]);
			echo "<span class='day'>" . date("l",$day) . "</span>\n";
			echo "<span class='conditions'>" . @$conditions[$i]->attributes()->{'weather-summary'} . "</span>\n";
			echo "<img class='icon' title='" . @$conditions[$i]->attributes()->{'weather-summary'} . "' src='{$icons[$i]}' />\n";
			echo "<span class='temperature'>";
			echo "<span class='hi'>" . $hitemps[$i] . "&deg;</span>";
			echo "<span class='low'>" . $lotemps[$i] . "&deg;</span>";
			echo "</span>\n";
			echo "</li>\n";
		}
		echo "</ul>\n";
		echo "</div>\n";
	}

	function displayForecast12($options){
		$periods = $options->get('num_periods12', 6);
		$m = $this->getModel();
		$id = $options->get("id",JRequest::getVar("id"));
		$f = $m->getForecast12($id);
		if(substr($f,0,6) != '<dwml '){
			echo $f;
			return;
		} 
		$xml = JFactory::getXML($f,false);
		$helper = new ZWeatherHelper();
		$days = $xml->data->{'time-layout'}[2]->{'start-valid-time'};
		$icons = $xml->data->parameters->{'conditions-icon'}->{'icon-link'};
		$conditions = $xml->data->parameters->weather->{'weather-conditions'};
		$temps = $xml->data->parameters->temperature;
		$hitemps = $temps[0]->value;
		$lotemps = $temps[1]->value;
		$alltemps = array();
		$starthigh = false;
		for($i = 0; $i < max(count($hitemps),count($lotemps));$i++){
			if($i == 0){
				if($helper->isMorning($helper->convertDate($days[$i]))){
					$starthigh=true;
					$alltemps[] = $hitemps[$i];
				}
				$alltemps[] = $lotemps[$i];
			}
			else {
				$alltemps[] = $hitemps[$i];
				$alltemps[] = $lotemps[$i];
			}
		}
		echo "<div class='forecast f12'>\n";
		echo "<ul>\n";
		for($i = 0; $i < $periods; $i++){
			$myday = $days[$i]->attributes()->{'period-name'};
			if(strlen($myday) == 0)
				$myday = date("l",$helper->convertDate($days[$i]));
			echo "<li class='p{$periods}'>\n";
			echo "<span class='day'>" . $myday . "</span>\n";
			echo "<span class='conditions'>" . $conditions[$i]->attributes()->{'weather-summary'} . "</span>\n";
			echo "<img class='icon' title='" . $conditions[$i]->attributes()->{'weather-summary'} . "' src='{$icons[$i]}' />\n";
			echo "<span class='temperature'>";
			if($starthigh){
				if(floor($i/2)*2 == $i)
					echo "<span class='hi'>hi " . $alltemps[$i] . "&deg;</span>";
				else
					echo "<span class='low'>low " . $alltemps[$i] . "&deg;</span>";
			}
			else {
				if(floor($i/2)*2 == $i)
					echo "<span class='low'>low " . $alltemps[$i] . "&deg;</span>";
				else
					echo "<span class='hi'>hi " . $alltemps[$i] . "&deg;</span>";				
			} 
			echo "</span>\n";
			echo "</li>\n";
		}
		echo "</ul>\n";
		echo "</div>\n";
	}

	function displayForecast3($options){
		$periods = $options->get('num_periods3', 8);
		$m = $this->getModel();
		$id = $options->get("id",JRequest::getVar("id"));
		$f = $m->getForecast3($id);
		if(substr($f,0,6) != '<dwml '){
			echo $f;
			return;
		} 
		$xml = JFactory::getXML($f,false);
		$helper = new ZWeatherHelper();
		$icons = $xml->data->parameters->{'conditions-icon'}->{'icon-link'};
		$xtemperature = $xml->data->parameters->temperature->value;
		$xtime = $xml->data->{'time-layout'}->{'start-valid-time'};
		$conditions = $xml->data->parameters->weather->{'weather-conditions'};
		echo "<div class='forecast f3'>\n";
		echo "<ul>\n";
		for($i = 0; $i < $periods; $i++){
			$weather = @$conditions[$i]->value->attributes()->{'weather-type'};
			$temperature = $xtemperature[$i];
			$ttime = ZWeatherHelper::convertDate($xtime[$i]);
			// Windows has some limitations on strftime format variables
			$time = strtolower(strftime("%I:%M %p",$ttime));
			if(substr($time,0,1) == "0")
				$time = substr($time,1);
			echo "<li class='p{$periods}'>\n";
			echo "<span class='time'>" . $time . "</span>\n";
			echo "<span class='conditions'>" . $weather . "</span>\n";
			echo "<img class='icon' title='" . $weather . "' src='{$icons[$i]}' />\n";
			echo "<span class='temperature'>{$temperature}&deg;</span>\n";
			echo "</li>\n";
		}
		echo "</ul>\n";
		echo "</div>\n";
	}

	function displayObservation($options){
		$m = $this->getModel();
		$id = $options->get("id",JRequest::getVar("id"));
		$o = $m->getObservation($id);
		$xml = JFactory::getXML($o,false);
		$helper = new ZWeatherHelper();
		//echo $o;
		$weather = $xml->weather;
		$location = $xml->location;
		$relative_humidity = $xml->relative_humidity;
		$temperature = $xml->temp_f;
		$observation_time = $xml->observation_time;
		$dewpoint = $xml->dewpoint_f;
		$windchill = $xml->windchill_f;
		$visibility = $xml->visibility_mi;
		$icon = $xml->icon_url_base . $xml->icon_url_name;
		$wind = $xml->wind_string;
		$pressure = $xml->pressure_string;
		$html = "";
		$html .= "<div class='observation'>";
		if($options->get("show_observation_title",0))
			$html .= "<h2>" . sprintf(JText::_('COM_ZWEATHER_CURRENT_OBSERVATION_FOR'),$location) . "</h2>\n";
		$html .= "<div><div class='summary'>";
		if($options->get("show_observation_weather",0))
			$html .= "<span class='weather'>$weather</span>";
		if($options->get("show_observation_icon",0))
			$html .= "<span class='icon'><img src='$icon' /></span>";
		if($options->get("show_observation_temperature",0))
			$html .= "<span class='temperature'>$temperature&deg;</span>";
		$html .= "</div>\n";
		$html .= "<div class='details'>\n";
		$p = array();
		if($options->get("show_observation_wind",0))
			$p[]=JText::_("COM_ZWEATHER_WIND") . ": " . $wind;
		if($options->get("show_observation_windchill",0))
			$p[]=JText::_("COM_ZWEATHER_WINDCHILL") . ": " . $windchill . "&deg;";
		if($options->get("show_observation_pressure",0))
			$p[]=JText::_("COM_ZWEATHER_PRESSURE") . ": " . $pressure;
		if($options->get("show_observation_dewpoint",0))
			$p[]=JText::_("COM_ZWEATHER_DEWPOINT") . ": " . $dewpoint;
		if($options->get("show_observation_relativehumidity",0))
			$p[]=JText::_("COM_ZWEATHER_RELATIVEHUMIDITY") . ": " . $relative_humidity . "%";
		$html .= "<p class='items'>" . implode(", ",$p) . "</p>\n";
		if($options->get("show_observation_time",0))
			$html .= "<p class='time'>" . $observation_time . "</p>\n";
		$html .= "</div></div>\n";
		$html .= "</div>\n";
		echo $html;
	}
	
	function displayAlert($options){
		
		$m = $this->getModel();
		$id = $options->get("id",JRequest::getVar("id"));
		$a = $m->getAlert($id);
		$a = str_replace(array('<cap:','</cap:'),array('<cap_','</cap_'),$a);
		if(strlen($a) == 0)
			return "";
		//echo $a; exit;
		$xml = JFactory::getXML($a,false);
		//echo "alert: $a";
		$html = "";
		$entries = $xml->entry;
		$get = $_GET;
		$get['layout'] = 'alert';
		$get['tmpl'] = 'component';
		$turl = array();
		foreach($get as $key => $value){
			$turl[]="$key=$value";
		}
		$url = JRoute::_("index.php?" . implode("&",$turl));
		$alertcount = 0;
		foreach($entries as $entry){
			$alertcount++;
			$title = $entry->{"cap_event"};
			$summary = $entry->{"summary"};
			//echo "**$event: $summary"; exit;
			if(strlen($title) > 0){
				$html .= "<div class='alert'>";
				$html .= "<span class='title' title='$summary'><a class='modal' href='$url#$alertcount' rel='{handler: \"iframe\", size: {x: 650, y: 400}}'>$title</a></span>";
				$html .= "</div>\n";
			}			
		}
		echo $html;
		return;
	}
	function displayAlertMessage($options){
		$m = $this->getModel();
		$id = $options->get("id",JRequest::getVar("id"));
		$a = $m->getAlert($id);
		$a = str_replace(array('<cap:','</cap:'),array('<cap_','</cap_'),$a);
		//echo $a; exit;
		$xml = JFactory::getXML($a,false);
		//echo "alert: $a";
		$html = "";
		$entries = $xml->entry;
		$alertcount = 0;
		foreach($entries as $entry){
			$alertcount++;
			$title = $entry->{"cap_event"};
			$url = $entry->id;
			$content = file_get_contents($url);
			$cxml = JFactory::getXML($content,false);
			$headline = $cxml->info->headline;
			$description = $cxml->info->description;
			//echo "**$event: $summary"; exit;
			if(strlen($headline) > 0){
				$html .= "<div class='alertinfo'>\n";
				$html .= "<a id='$alertcount' ><h1>$headline</h1></a>\n";
				$html .= "<pre class='description'>$description</pre>\n";
				$html .= "</div>";
			}
			else {
				$html .= JText::_("COM_ZWEATHER_NO_ALERTS_AT_THIS_TIME");
			}
		}
		if($alertcount > 0){
			$phtml = '<ul class="actions"><li><a href="#" onclick="window.print();return false;"><img src="media/system/images/printButton.png" alt="Print"  /></a></li></ul>';
			$html = $phtml . $html;
		}
		echo $html;
		return;
	}
}
